#!/usr/bin/perl

#################################################################
# isr_srv_ls.pl - List information about a user's parcels
#
# $Id: isr_srv_ls.pl,v 1.15 2004/11/05 18:52:39 droh Exp $
#################################################################

#
#                  Internet Suspend/Resume (Release 0.5)
#           A system for capture and transport of PC state
#
#            Copyright (c) 2004, Carnegie Mellon University
#              Copyright (c) 2002-2004, Intel Corporation
#                         All Rights Reserved
#
# This software is distributed under the terms of the Eclipse Public
# License, Version 1.0 which can be found in the file named LICENSE.
# ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS SOFTWARE CONSTITUTES
# RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT
#

##########
# Prologue
##########
use strict;
use Getopt::Std;
use File::stat;
use Term::ANSIColor qw(:constants);
use Socket;
use Sys::Hostname;
use lib "/usr/local/isr/bin";
use Server;
$| = 1; # Autoflush output on every print statement

####################
# begin main routine
####################

# Variables
my $userdir;
my $username;
my $parcel;
my $unlocked;
my $logentry;
my $unused;
my $action;
my $client;
my $date;
my $user;
my $state;
my $longflag;
my $version;
my $size;
my $line;
my $hostname = hostname();

my @versions;

#
# Parse the command line args
#
no strict 'vars';
getopts('hu:p:l');

if ($opt_h) {
    usage();
}
if (!$opt_u) {
    usage("Missing user name (-u)");
}
if (!$opt_p) {
    usage("Missing parcel name (-p)");
}
$longflag = $opt_l;
$username = $opt_u;
$parcel = $opt_p;
$userdir = "$Server::CONTENT_ROOT" . "/$username";
use strict 'vars';

#
# Use only the hostname portion of this hosts fqdn
#
$hostname =~ /^(\w+)\.?/; # at least 1 alphanum char followed by 0 or 1 dots
$hostname = $1;

#
# Determine the last time this parcel was acquired or released
#
system("rm -f $userdir/entry");
$unlocked = system("$Server::SRVBIN/isr_srv_lock.pl -p $username/$parcel -n foo -Vc > $userdir/entry");
$logentry = `cat $userdir/entry`;
chomp($logentry);
system("rm -f $userdir/entry");

#
# Determine it's present state
# 
if ($unlocked) {
    $state = "released";
}
else {
    $state = "acquired";
}

#
# Parse the log entry
#
($unused, $date, $action, $user, $unused, $client, $unused) = 
    split('\|', $logentry);
if ($logentry and 
    (($unlocked and $action ne "released") or
    (!$unlocked and $action ne "acquired")) ) {
    errexit("System error: inconsistent log entry: unlocked=$unlocked action=$action\nlogentry=$logentry");
}

#
# We don't need the day of the week on the date
#
$date =~ s/^\w+\s//;

#
# Print the main output line
#
if ($logentry and $unlocked) {
    print GREEN;
    printf("%s [%s] %s by %s on %s\n", $parcel, $hostname, 
	   $state, $client, $date);
    print RESET;
}
elsif ($logentry and !$unlocked) {
    print RED;
    printf("%s [%s] %s by %s on %s\n", $parcel, $hostname, 
	   $state, $client, $date);
    print RESET;
}
else {
    print("$parcel [$hostname] has never been checked out.\n");
}

# 
# If the user wants to see the available versions of the parcel, print 
# those too.
#
if ($longflag) {
    opendir(DIR, "$userdir/$parcel")
	or unix_errexit("Could not open directory $userdir/$parcel");
    @versions = sort grep(/^\d+$/, readdir(DIR)); # numbers only
    closedir(DIR);
    foreach $version (@versions) {
	if (-e "$userdir/$parcel/$version/keyring.enc") {
	    $date = localtime(stat("$userdir/$parcel/$version/keyring.enc")->mtime);
	}
	else {
	    $date = "[not available]";
	}

	$line = `du -h -s $userdir/$parcel/$version`;
	($size, $unused) = split(" ", $line);
	printf("  %s %6s  %s\n", $version, $size, $date);
    }


    exit 0;
}


############################################
# usage - print help message and terminate #
############################################

sub usage
{
    my $msg = shift;
    my $progname;

    # Strip any path information from the program name
    ($progname = $0) =~ s#.*/##s; 
    
    if ($msg) {
        print "$progname: $msg\n\n";
    }
    
    print "Usage: $progname [-hl] -u <username> -p <parcel>\n";
    print "Options:\n";
    print "  -h              Print this message\n";
    print "  -u <username>   User name\n";  
    print "  -p <parcel>     Parcel name\n";  
    print "  -l              List all the available versions\n";  
    print "\n";
    exit 0;
}
